#include <unity/scopes/Runtime.h>
#include <string>
#include <boost/filesystem.hpp>
#include <iostream>
#include "scope.h"

int main(int argc, char **argv)
{
    if (argc != 3) {
        std::cerr << "Invalid arguments. Expected runtime config file and scope id" << std::endl;
        return 1;
    }

    const std::string runtime_cfg(argv[1]);
    const std::string scope_config(argv[2]);

    auto const scope_config_path = boost::filesystem::canonical(scope_config);
    auto const scope_id = scope_config_path.stem().native();

    try
    {
        auto rt = unity::scopes::Runtime::create_scope_runtime(scope_id, runtime_cfg);
        unity::scopes::ScopeBase *scope = new scope::Scope();
        rt->run_scope(scope, scope_config);
        return 0;
    }
    catch (const std::exception &e)
    {
        std::cerr << "Failed to load scope " << scope_id << ": " << e.what();
    }
    return 1;
}
