#include "query.h"

#include <unity/scopes/CategorisedResult.h>
#include <unity/scopes/CategoryRenderer.h>
#include <unity/scopes/QueryBase.h>
#include <unity/scopes/SearchReply.h>

namespace sc = unity::scopes;

using namespace std;
using namespace scope;

Query::Query(const Scope& scope, const sc::CannedQuery &query, const sc::SearchMetadata &metadata) :
    sc::SearchQueryBase(query, metadata),
    scope_(scope) {
}

void Query::cancelled() {
}

void Query::generate_results(sc::SearchReplyProxy const& reply)
{
    for (auto const &def: scope_.category_defs) {
        sc::Category::SCPtr cat = reply->register_category(def.id, def.name, "", sc::CategoryRenderer(def.renderer));

        for (int i = 0; i<def.result_count; i++) {
            auto const countstr = std::to_string(i);
            sc::CategorisedResult res(cat);
            for (auto const& val: def.result_values) {
                // substitue %COUNT% and %SCOPE_DIR% tokens
                res[val.first] = subst(val.second, i);
            }

            if (!reply->push(res)) {
                return;
            }
        }
    }
}

void Query::run(sc::SearchReplyProxy const& reply) {
    try {
        generate_results(reply);

    } catch (std::exception &e) {
        cerr << e.what() << endl;
        reply->error(current_exception());
    }
}

void Query::replace_all(std::string &data, const std::string &old_value, const std::string &new_value) {
    size_t pos = 0;
    while (true) {
        pos = data.find(old_value, pos);
        if (pos == std::string::npos) {
            break;
        }
        data.replace(pos, old_value.size(), new_value);
    }
}

std::string Query::subst(const std::string& pattern, int count) const
{
    static const std::string SCOPE_DIR_TOKEN = "%SCOPE_DIR%";
    static const std::string COUNT_TOKEN = "%COUNT%";
    std::string uri = pattern;
    replace_all(uri, SCOPE_DIR_TOKEN, scope_.scope_dir);
    replace_all(uri, COUNT_TOKEN, std::to_string(count + 1));
    return uri;
}

sc::Variant Query::subst(const sc::Variant& input, int count) const
{
    switch (input.which()) {
        case sc::Variant::String:
            return sc::Variant(subst(input.get_string(), count));
        case sc::Variant::Dict:
            {
                sc::VariantMap out;
                for (auto const& kv: input.get_dict()) {
                    out[kv.first] = subst(kv.second, count);
                }
            }
        case sc::Variant::Array:
            {
                sc::VariantArray out;
                for (auto const &val: input.get_array()) {
                    out.push_back(subst(val, count));
                }
                return sc::Variant(out);
            }
            break;
        default:
            break;
    }
    return input;
}
